"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.authenticate = void 0;
const msal_node_1 = require("@azure/msal-node");
const config_1 = require("./config");
const msal_node_extensions_1 = require("@azure/msal-node-extensions");
let client;
function getClient(config) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!client) {
            const persistence = yield msal_node_extensions_1.FilePersistence.create("./tokenCache.json");
            client = new msal_node_1.PublicClientApplication({
                auth: {
                    authority: "https://login.microsoftonline.com/common",
                    clientId: config.clientId,
                },
                cache: {
                    cachePlugin: new msal_node_extensions_1.PersistenceCachePlugin(persistence),
                },
            });
        }
        return client;
    });
}
function authenticate(config) {
    return __awaiter(this, void 0, void 0, function* () {
        const client = yield getClient(config);
        const accountInfo = yield client.getTokenCache().getAllAccounts();
        return yield client
            .acquireTokenSilent({
            scopes: config_1.SCOPES,
            account: accountInfo[0],
        })
            .catch(() => {
            if (!client)
                return null;
            return client.acquireTokenByDeviceCode({
                scopes: config_1.SCOPES,
                deviceCodeCallback: (res) => {
                    console.log(res.message);
                },
            });
        });
    });
}
exports.authenticate = authenticate;
