"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _Content_document;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Content = void 0;
const node_html_parser_1 = require("node-html-parser");
/**
 * List of invalid attributes we should remove part of our
 * sanitizer.
 */
const invalidAttributes = ["lang"];
/**
 * This list includes attributes we want to further operate
 * on but which should otherwise be left alone.
 */
const validAttributes = ["style", "data-tag"];
/**
 * HTML output from Evernote is relatively clean but there
 * are a lot of domain-specific inline styles. This list serves
 * as a very basic inline styles sanitizer.
 */
const validStyles = [
    "background-color",
    "color",
    "text-align",
    "font-family",
    "font-size",
];
/**
 * This is a list of special elements used by Evernote for different
 * purposes.
 */
const invalidElements = ["object", "img"];
const cssSelector = [
    ...validAttributes.map((attr) => `[${attr}]`),
    ...invalidAttributes.map((attr) => `[${attr}]`),
    ...invalidElements,
].join(",");
class Content {
    constructor(html, options) {
        this.options = options;
        _Content_document.set(this, void 0);
        __classPrivateFieldSet(this, _Content_document, (0, node_html_parser_1.parse)(html), "f");
    }
    transform(handler) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const body = __classPrivateFieldGet(this, _Content_document, "f").querySelector("body");
            const elements = body === null || body === void 0 ? void 0 : body.querySelectorAll(cssSelector);
            if (!body || !elements)
                return "";
            for (let element of elements) {
                if (!element)
                    continue;
                let elementType = filterAttributes(element) || element.tagName.toLowerCase();
                switch (elementType) {
                    case "checklist":
                        let siblings = [];
                        const ul = __classPrivateFieldGet(this, _Content_document, "f").createElement("ul");
                        ul.classList.add("checklist");
                        let next = element;
                        while (next && ((_a = next.getAttribute("data-tag")) === null || _a === void 0 ? void 0 : _a.startsWith("to-do"))) {
                            const li = __classPrivateFieldGet(this, _Content_document, "f").createElement("li");
                            li.innerHTML = next.innerHTML;
                            if (next.getAttribute("data-tag") === "to-do:completed")
                                li.classList.add("checked");
                            ul.appendChild(li);
                            if (next !== element)
                                siblings.push(next);
                            next = next.nextElementSibling;
                        }
                        element.replaceWith(ul);
                        siblings.forEach((elem) => {
                            elem.removeAttribute("data-tag");
                            elem.remove();
                        });
                        break;
                    case "img":
                    case "object": {
                        const data = yield downloadAttachment(element, ["data-fullres", "src", "data"], this.options.attachmentResolver);
                        if (!data) {
                            element.remove();
                            break;
                        }
                        element.setAttribute("data", data);
                        if (handler) {
                            const result = yield handler.process(elementType, element);
                            if (!!result)
                                element.replaceWith(result);
                            else
                                element.remove();
                        }
                        else
                            element.remove();
                        break;
                    }
                }
            }
            return body.innerHTML;
        });
    }
    get raw() {
        var _a;
        return (((_a = __classPrivateFieldGet(this, _Content_document, "f").querySelector("body")) === null || _a === void 0 ? void 0 : _a.outerHTML) ||
            __classPrivateFieldGet(this, _Content_document, "f").outerHTML);
    }
    toJSON() {
        return this.raw;
    }
}
exports.Content = Content;
_Content_document = new WeakMap();
function filterAttributes(element) {
    let elementType = null;
    for (let attr of invalidAttributes) {
        if (element.hasAttribute(attr))
            element.removeAttribute(attr);
    }
    for (let attr of validAttributes) {
        if (!element.hasAttribute(attr))
            continue;
        const value = element.getAttribute(attr);
        if (!value) {
            element.removeAttribute(attr);
            continue;
        }
        switch (attr) {
            case "data-tag":
                if (!value.startsWith("to-do"))
                    break;
                elementType = "checklist";
                break;
            case "style": {
                const styles = stylesToObject(value);
                for (let style in styles) {
                    if (validStyles.indexOf(style) === -1)
                        delete styles[style];
                }
                const newStyle = objectToStyles(styles);
                if (newStyle)
                    element.setAttribute(attr, newStyle);
                else
                    element.removeAttribute(attr);
                break;
            }
        }
    }
    return elementType;
}
function stylesToObject(input) {
    const styles = input.split(";");
    const output = {};
    for (let style of styles) {
        const [key, value] = style.trim().split(":");
        output[key] = value;
    }
    return output;
}
function objectToStyles(input) {
    const output = [];
    for (let key in input) {
        output.push(`${key}:${input[key]}`);
    }
    return output.join(";");
}
function downloadAttachment(element, attributes, resolver) {
    return __awaiter(this, void 0, void 0, function* () {
        let src = null;
        for (let attribute of attributes) {
            src = element.getAttribute(attribute);
            if (src)
                break;
        }
        if (!src)
            return false;
        const data = yield resolver(src);
        if (!data)
            throw new Error(`Failed to download attachment at ${src}.`);
        return data.toString("base64");
    });
}
